#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 3.0e-16
#define RNMX (1.0-EPS)

extern long idum2, iy, iv[NTAB];

double ran2(long *idum)
   {
      int j;
      long k;
      double temp;

      if(*idum <= 0)
         {
            if(-*idum < 1) *idum = 1;
            else *idum = -*idum;

            idum2 = *idum;

            for(j = NTAB + 7; j >= 0; j--)
               {
                  k = *idum / IQ1;
                  *idum = IA1 * (*idum - k * IQ1) - k * IR1;
                  if(*idum < 0) *idum += IM1;
                  if(j < NTAB) iv[j] = *idum;
               }

            iy = iv[0];
         }

      k = *idum / IQ1;
      *idum = IA1 * (*idum - k * IQ1) - k * IR1;
      if(*idum < 0) *idum += IM1;
      k = idum2 / IQ2;
      idum2 = IA2 * (idum2 - k * IQ2) - k * IR2;
      if(idum2 < 0) idum2 += IM2;
      j = iy / NDIV;
      iy = iv[j] - idum2;
      iv[j] = *idum;
      if(iy < 1) iy += IMM1;

      if((temp = AM * iy) > RNMX) return RNMX;
      else return temp;
   }

#undef NTAB
/* (C) Copr. 1986-92 Numerical Recipes Software m6?!+!$:)#. */

#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#define NR_END 1
#define FREE_ARG char*

void nrerror(char *error_text)
/* Numerical Recipes standard error handler */
   {
      fprintf(stderr, "Numerical Recipes run-time error...\n");
      fprintf(stderr, "%s\n",error_text);
      fprintf(stderr, "...now exiting to system...\n");
      exit(EXIT_FAILURE);
}

int *ivector(long nl, long nh)
/* allocate an int vector with subscript range v[nl..nh] */
   {
      int *v;

      v = (int *)malloc((size_t)
          ((nh - nl + 1 + NR_END) * sizeof(int)));

      if (!v) nrerror("allocation failure in ivector()");
      return v - nl + NR_END;
}

void free_ivector(int *v, long nl, long nh)
/* free an int vector allocated with ivector() */
   {
      free((FREE_ARG) (v + nl - NR_END));
   }

#define SWAP(a,b) temp = (a); (a) = (b); (b) = temp;
#define M 7
#define NSTACK 50

void sort2(unsigned long n, double *arr, long *brr)
   {
      unsigned long i, ir = n, j, k, l = 1;
      int *istack, jstack = 0;
      double a, b, temp;

      istack = ivector(1, NSTACK);

      for( ; ; )
         {
            if(ir - l < M)
               {
                  for(j = l + 1; j <= ir; j++)
                     {
                        a = arr[j];
                        b = brr[j];

                        for(i = j - 1; i >= 1; i--)
                           {
                              if (arr[i] <= a) break;
                              arr[i + 1] = arr[i];
                              brr[i + 1] = brr[i];
                           }

                        arr[i + 1] = a;
                        brr[i + 1] = b;
                     }

                  if (!jstack)
                     {
                        free_ivector(istack, 1, NSTACK);
                        return;
                     }

                  ir = istack[jstack];
                  l = istack[jstack - 1];
                  jstack -= 2;
               }

            else
               {
                  k = (l + ir) >> 1;
                  SWAP(arr[k], arr[l + 1])
                  SWAP(brr[k], brr[l + 1])

                  if(arr[l + 1] > arr[ir])
                     {
                        SWAP(arr[l + 1], arr[ir])
                        SWAP(brr[l + 1], brr[ir])
                     }

                  if(arr[l] > arr[ir])
                     {
                        SWAP(arr[l], arr[ir])
                        SWAP(brr[l], brr[ir])
                     }

                  if(arr[l + 1] > arr[l])
                     {
                        SWAP(arr[l + 1], arr[l])
                        SWAP(brr[l + 1], brr[l])
                     }

                  i = l + 1;
                  j = ir;
                  a = arr[l];
                  b = brr[l];

                  for( ; ; )
                     {
                        do i++; while (arr[i] < a);
                        do j--; while (arr[j] > a);
                        if(j < i) break;
                        SWAP(arr[i], arr[j])
                        SWAP(brr[i], brr[j])
                     }

                  arr[l] = arr[j];
                  arr[j] = a;
                  brr[l] = brr[j];
                  brr[j] = b;
                  jstack += 2;

                  if(jstack > NSTACK)
                     nrerror("NSTACK too small in sort2.");

                  if(ir - i + 1 >= j - l)
                     {
                        istack[jstack] = ir;
                        istack[jstack - 1] = i;
                        ir = j - 1;
                     }

                  else
                     {
                        istack[jstack] = j - 1;
                        istack[jstack - 1] = l;
                        l = i;
                     }
               }
         }
   }

#undef M
#undef NSTACK
#undef SWAP
#undef NRANSI
/* (C) Copr. 1986-92 Numerical Recipes Software m6?!+!$:)#. */
